﻿using ProjectAccordionBus.CollectionGenericObjects;
using ProjectAccordionBus.Drawnings;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAccordionBus;

/// <summary>
/// 
/// </summary>
public partial class FormBusCollection : Form
{
    /// <summary>
    /// 
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBusCollection()
    {
        InitializeComponent();
    }
    /// <summary>
    /// Добавление улучшенного троллейбуса
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddAccordionBus_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningAccordionBus));

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создания объекта</param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }

        Random random = new();
        DrawningBus drawningBus;
        switch (type)
        {
            case nameof(DrawningBus):
                drawningBus = new DrawningBus(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningAccordionBus):
                drawningBus = new DrawningAccordionBus(random.Next(100, 300), random.Next(1000, 3000), GetColor(random), GetColor(random),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        if (_company + drawningBus != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }

        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
    /// Выбор цвета
    /// </summary>
    /// <param name="random"></param>
    /// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }

        return color;
    }

    private void buttonAddBus_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningBus));
    }

    private void buttonAddAccordionBus_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningAccordionBus));
    }

    private void buttonDeleteBus_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удалён");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null) return;

        DrawningBus? bus = null;
        int counter = 100;
        while (bus == null || counter > 0)
        {
            bus = _company.GetRandomObject();
            counter--;
        }

        if (bus == null) return;

        FormAccordionBus form = new()
        {
            SetBus = bus
        };
        form.ShowDialog();
    }

    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    private void comboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new BusSharingService(pictureBox.Width, pictureBox.Height, new MassiveGenericObjects<DrawningBus>());
                break;
        }
    }

}
